/******************************************************************************
Author:         Dave Szybala
Website:        www.davidszybala.com
Alt Website:    www.twistedgerbil.com
Date:           6/13/2015
Program:        material_break.ms
Description:    This script will break apart geometry with a multisub material
                applied
******************************************************************************/

try (destroyDialog material_break_rollout) catch()


struct MaterialBreaker
(
    all_materials = for material in sceneMaterials collect material,
    all_material_names = for material in all_materials collect material.name,
    
    /**************************************************************************
    Returns a unique list of face material ids for the 'current_poly' object
    **************************************************************************/
    fn get_poly_face_ids current_poly =
    (
        local poly_id_list = #()
        
        for i = 1 to (polyop.getNumFaces current_poly) do
            appendIfUnique poly_id_list (polyop.getFaceMatID current_poly i)
        
        sort poly_id_list
        poly_id_list --return value
    ), --end fn
    
    /**************************************************************************
    Returns a unique list of face material ids for the 'current_mesh' object
    **************************************************************************/
    fn get_mesh_face_ids current_mesh =
    (
        local mesh_id_list = #()
        
        for i = 1 to (meshop.getNumFaces current_mesh) do
            appendIfUnique mesh_id_list (getFaceMatID current_mesh i)
        
        sort mesh_id_list
        mesh_id_list --return value
    ), --end fn
    
    /**************************************************************************
    Copies the 'current_poly' object and deletes all faces with material ids
    in the list 'delete_ids' Returns the new node
    **************************************************************************/
    fn delete_poly_by_ids current_poly delete_ids =
    (
        maxOps.CloneNodes current_poly newNodes:&new_poly
        polyop.setFaceSelection new_poly[1] #none
        
        for id in delete_ids do
            new_poly[1].selectByMaterial id clearCurrentSelection:false
        
        polyop.deleteFaces new_poly[1] #selection
        new_poly[1] --return value
    ), --end fn
    
    /**************************************************************************
    Copies the 'current_mesh' object and deletes all faces with material ids
    in the list 'delete_ids' Returns the new node
    **************************************************************************/
    fn delete_mesh_by_ids current_mesh delete_ids =
    (
        local delete_faces = #()
        maxOps.CloneNodes current_mesh newNodes:&new_mesh
        
        for i = 1 to (meshop.getNumFaces current_mesh) do
        (
            id = getFaceMatID current_mesh i
            if (findItem delete_ids id) > 0 do
                append delete_faces i
        ) --end for
        
        meshop.deleteFaces new_mesh[1] delete_faces
        new_mesh[1] --return value
    ), --end fn
    
    /**************************************************************************
    Returns a standard material (new or a reference if it already exists) used
    to mark objects with no material assignment
    **************************************************************************/
    fn get_null_material =
    (
        local null_material = undefined
        local loop_control = true
        
        for current_material in all_materials while loop_control do
        (
            if current_material.name == "NO_MATERIAL_ASSIGNMENT" do
            (
                null_material = current_material
                loop_control = false
            ) --end if
        ) --end for
        
        if loop_control do
        (
            null_material = StandardMaterial name:"NO_MATERIAL_ASSIGNMENT"
            null_material.Diffuse = color 255 0 255
            append all_materials null_material
            append all_material_names "NO_MATERIAL_ASSIGNMENT"
        ) --end if
        
        null_material --return value
    ), --end fn
    
    /**************************************************************************
    Assigns the sub material as the base material for the new node
    **************************************************************************/
    fn assign_material current_object material_id =
    (
        local material_list = current_object.material.materialList
        local material_id_list = current_object.material.materialIDList
        
        id_index = findItem material_id_list material_id
        
        if id_index > 0 then
        (
            if material_list[id_index] != undefined then
            (
                name = material_list[id_index].name
                name_index = findItem all_material_names name
                
                if name_index > 0 then
                    current_object.material = all_materials[name_index]
                else
                (
                    new_material = copy material_list[id_index]
                    append all_materials new_material
                    append all_material_names new_material.name
                    current_object.material = new_material
                ) --end else
            ) --end if
            else
                current_object.material = this.get_null_material()
        ) --end if
        else
            current_object.material = this.get_null_material()
    ), --end fn
    
    /**************************************************************************
    Breaks the 'current_poly' object by material ids
    **************************************************************************/
    fn break_poly_object current_poly =
    (
        local poly_ids = this.get_poly_face_ids(current_poly)
        
        for i = 1 to poly_ids.count do
        (
            delete_ids = copy poly_ids #nomap
            deleteItem delete_ids i
            new_poly = this.delete_poly_by_ids current_poly delete_ids
            
            new_poly.name = (current_poly.name + "_MatID_" + (poly_ids[i] as string))
            
            this.assign_material new_poly poly_ids[i]
        ) --end for
        
        delete current_poly
    ), --end fn
    
    /**************************************************************************
    Breaks the 'current_mesh' object by material ids
    **************************************************************************/
    fn break_mesh_object current_mesh =
    (
        local mesh_ids = this.get_mesh_face_ids(current_mesh)
        
        for i = 1 to mesh_ids.count do
        (
            delete_ids = copy mesh_ids #nomap
            deleteItem delete_ids i
            new_mesh = this.delete_mesh_by_ids current_mesh delete_ids
            
            new_mesh.name = (current_mesh.name + "_MatID_" + (mesh_ids[i] as string))
            
            this.assign_material new_mesh mesh_ids[i]
        ) --end for
        
        delete current_mesh
    ) --end fn
) --end struct


( --local scope
    global material_break_rollout
    local material_break_ini = getDir #plugcfg + "\\MaterialBreak.ini"
    local material_break_position

    rollout material_break_rollout "Material Break"
    (
        groupbox option_group "Options" pos:[5,5] width:200 height:40
            checkbox selection_checkbox "Selection Only" pos:[15,25] width:100 height:15 checked:true
            
        progressbar progress_bar pos:[5,55] width:200 height:10
        button break_button "Break Multi/Sub-Objects" pos:[5,70] width:200 height:20
        label status_label "INFO: Standing by..." pos:[5,97] width:200 height:15
        
        /**********************************************************************
        UI move event handler. Stores the curent position of the UI
        **********************************************************************/
        on material_break_rollout moved pos do
            setIniSetting material_break_ini "Dialog" "Position" (pos as string)
            
        /**********************************************************************
        Break button event handler. Executes the script
        **********************************************************************/
        on break_button pressed do
        (
            local breaker = MaterialBreaker()
            local break_objects = #()
            local error_counter = 0
            local percent
            
            progress_bar.value = 0
            progress_bar.color = [200,0,0]
            
            if selection_checkbox.checked then 
                break_objects = selection as array
            else
                break_objects = geometry as array
                
            if break_objects.count == 0 do
            (
                status_label.text = "ERROR: No geometry selected or found"
                return false
            ) --end if
                
            status_label.text = "INFO: Breaking multi/sub objects..."
            
            with redraw off
            (
                max create mode
                clearSelection()
                
                for i = 1 to break_objects.count do
                (
                    if (classOf break_objects[i].material) == Multimaterial do
                    (
                        collapseStack break_objects[i]
                        
                        case (classOf break_objects[i]) of
                        (
                            (Editable_Poly): breaker.break_poly_object break_objects[i]
                            (Editable_mesh): breaker.break_mesh_object break_objects[i]
                            default: (
                                        message = "ERROR: Could not break apart "
                                        message += break_objects[i].name
                                        print(message)
                                        error_counter += 1
                                     ) --end default
                        ) --end case
                    ) --end if
                    
                    percent = 100.0 * i/break_objects.count
                    progress_bar.value = percent
                    if ((maxVersion())[1] / 1000) > 12 do windows.processPostedMessages()
                ) --end for i
            ) --end redraw
            
            progress_bar.color = [0,200,0]
            progress_bar.value = 100
            
            if error_counter == 0 then
                status_label.text = "INFO: Material break complete"
            else
                status_label.text = ("WARNING: " + (error_counter as string) + " error(s). Check listener")
        ) --end on
    ) --end rollout

    material_break_position = execute (getIniSetting material_break_ini "Dialog" "Position")
    if material_break_position == OK do material_break_position = [100,100]
    if material_break_position.x > sysinfo.desktopsize.x - 210 do
        material_break_position.x = sysinfo.desktopsize.x - 215
    if material_break_position.y > sysinfo.desktopsize.y - 170 do
        material_break_position.y = sysinfo.desktopsize.y - 170
    if material_break_position.x < 0 do material_break_position.x = 0 
    if material_break_position.y < 0 do material_break_position.y = 0
    createDialog material_break_rollout 210 110 material_break_position.x material_break_position.y
) --end local scope
